@@ -2,7 +2,7 @@ package WWW::Mechanize::Plugin::AutoWrite;
 
 =head1 NAME
 
-WWW::Mechanize::Plugin::AutoWrite - WWW::Mechanize plugin that writes the fetched pages to the disk
+WWW::Mechanize::Plugin::AutoWrite - WWW::Mechanize plugin that writes the fetched pages to the disk.
 
 =head1 SYNOPSIS
 
@@ -10,7 +10,7 @@ WWW::Mechanize::Plugin::AutoWrite - WWW::Mechanize plugin that writes the fetche
 	use WWW::Mechanize::Plugin::AutoWrite;
 	
 	my $mech = WWW::Mechanize->new();
-	$mech->autowrite('/tmp/mech.html');
+	$mech->autowrite->file('/tmp/mech.html');
 	
 	$mech->get('http://search.cpan.org/');
 	# now the contents of the page is written to /tmp/mech.html
@@ -18,7 +18,7 @@ WWW::Mechanize::Plugin::AutoWrite - WWW::Mechanize plugin that writes the fetche
 or:
 
 	my $mech = WWW::Mechanize->new();
-	$mech->autowritedir('/tmp/mech/');
+	$mech->autowrite->dir('/tmp/mech/');
 	
 	$mech->get('http://search.cpan.org/');
 	# now the contents of the page are written to /tmp/mech/001.html and the HTTP
@@ -37,8 +37,8 @@ or:
 or:
 
 	my $mech = WWW::Mechanize->new();
-	$mech->autowritedir('/tmp/mech/');       # Save the whole session
-	$mech->autowrite('/tmp/mech/last.html'); # Save the last page in it's own file
+	$mech->autowrite->dir('/tmp/mech/');           # Save the whole session
+	$mech->autowrite->file('/tmp/mech/last.html'); # Save the last page in a file
 	
 	$mech->get('http://www.wikipedia.com/');
 	# now the contents of the page are written both to /tmp/mech/001.html and 
@@ -76,6 +76,38 @@ complex web sites.
 
 This module can be configured through the attributes enumerated here.
 
+=head2 file
+
+Get/set the name of the file where the last page downloaded (the content's of
+the HTTP response) will be saved.
+
+Set this attribute to a false value in order to disable the saving of the last
+page downloaded.
+
+=head2 dir
+
+Get/set the name of the folder where the HTTP session (the content's of the HTTP
+response as well as the HTTP headers) will be saved.
+
+Set this attribute to a false value in order to disable the saving of the HTTP
+session.
+
+=head2 counter
+
+Get/set the counter used no name each file with a unique name when saving the
+HTTP session (the counter is used only when L</dir> is set).
+
+
+It can be useful to reset the counter when multiple sessions need to be saved
+into different folders.
+
+	foreach my $entry (@entries) {
+		$mech->autowrite->dir("/tmp/$entry/");
+		$mech->autowrite->counter(0);
+		# Complex mechanize		
+		mechanize_process($mech, $entry);
+	}
+
 =cut
 
 
@@ -83,7 +115,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 use File::Slurp qw{ write_file };
 use File::Path qw{ mkpath };
@@ -92,11 +124,20 @@ use File::Basename qw{ fileparse };
 
 use MIME::Types;
 
+use base qw(Class::Accessor::Fast);
+__PACKAGE__->mk_accessors(
+	qw(
+		file
+		dir
+		counter
+	)
+);
 
-# We need a reference to the original method used for the requests.
+
+# We need a reference to the original method used by mechanize for the requests.
 my $REQUEST_SUB;
 BEGIN {
-	 $REQUEST_SUB = \&WWW::Mechanize::request;
+	$REQUEST_SUB = \&WWW::Mechanize::request;
 }
 
 
@@ -104,134 +145,56 @@ BEGIN {
 my $MIME_TYPES = MIME::Types->new();
 
 
-=head2 autowrite
-
-Get/set the name of the file where the last page downloaded (the content's of
-the HTTP response) will be saved.
-
-Set this attribute to a false value in order to disable the saving of the last
-page downloaded.
-
-=cut
-
-sub WWW::Mechanize::autowrite {
-	my $self = shift;
-
-	# set
-	if (@_) {
-		$self->{'autowrite'} = shift;
-	}
-	# get
-	else {
-		return $self->{'autowrite'};
-	}
-}
-
-
-=head2 autowritedir
-
-Get/set the name of the folder where the HTTP session(the content's of the HTTP
-response as well as the HTTP headers) will be saved.
-
-Set this attribute to a false value in order to disable the saving of the HTTP
-session.
-
-=cut
-
-sub WWW::Mechanize::autowritedir {
-	my $self = shift;
-
-	# set
-	if (@_) {
-		$self->{'autowritedir'} = shift;
-	}
-	# get
-	else {
-		return $self->{'autowritedir'};
-	}
-}
-
 =head1 METHODS
 
 This module offers the following public methods:
 
 =cut
 
-=head2 autowritedir_reset_counter
-
-Resets the counter used by autowritedir. This only affects the outcome of
-L</autowritedir>.
-
-This can be useful when multiple sessions need to be saved into different
-folders.
-
-	foreach my $entry (@entries) {
-		$mech->autowritedir("/tmp/$entry/");
-		$mech->autowritedir_reset_counter;
-		# Complex mechanize		
-		mechanize_process($mech, $entry);
-	}
-
-=cut
-
-sub WWW::Mechanize::autowritedir_reset_counter {
-	my $self = shift;
-	$self->{'autowritedir_counter'} = 0;
-}
-
 
 #
 # Returns the next iteration of the counter. This method initializes the counter
 # the first time it's invoked.
 #
-sub WWW::Mechanize::_autowritedir_inc_counter {
+sub _inc_counter {
 	my $self = shift;
 	
-	if (! defined $self->{'autowritedir_counter'}) {
-		$self->{'autowritedir_counter'} = 0;
+	if (! defined $self->{'counter'}) {
+		$self->{'counter'} = 0;
 	}
 	
-	return ++$self->{'autowritedir_counter'};
+	return ++$self->{'counter'};
 }
 
 
-no warnings qw{ redefine };
+=head2 write_to_file
 
-sub WWW::Mechanize::request {
+This method writes the HTTP requests into a file and/or a folder. It's called
+automatically by mechanize once the plugin is loaded and configured.
+
+=cut
+
+sub write_to_file {
 	my $self = shift;
-	my ($request, @args) = @_;
-	
-	# Perform the actual HTTP request
-	my $response = $REQUEST_SUB->($self, $request, @args);
-
-	# Write the request, response and contents
-	_write_to_file($self, $request, $response);
-	
-	return $response;
-}
-
-
-sub _write_to_file {
-	my $self = shift;
-	my ($request, $response) = @_;
+	my ($mech, $request, $response) = @_;
 
 
 	# write to a single file if autowrite is set
-	if (my $filename = $self->autowrite) {
+	if (my $filename = $self->file) {
 		# Make sure that the path to the file exists
 		my (undef, $folder) = fileparse($filename);
 		mkpath($folder);
 		
-		write_file($filename, $self->content);
+		write_file($filename, $mech->content);
 	}
 
 
 	# write to multiple files in a folder if autowritedir is set
-	if (my $foldername = $self->autowritedir) {
+	if (my $foldername = $self->dir) {
 		mkpath($foldername);    # works fine with already existing folders
 		# my $encoding = $response->content_encoding;
 
-		my $counter = $self->_autowritedir_inc_counter;
+		my $counter = $self->_inc_counter();
 		my $file;
 		
 		# Get the extension of the file based on the mime-type
@@ -245,7 +208,7 @@ sub _write_to_file {
 			$foldername,
 			sprintf "%03d.%s", $counter, $extension
 		);
-		write_file($file, $self->content);
+		write_file($file, $mech->content);
 
 	
 		# Remember that the response has the document body which we don't want at
@@ -264,8 +227,105 @@ sub _write_to_file {
 }
 
 
+#
+# NOTE: We are injecting methods into WWW::Mechanize it's evil but it's the only
+#       way for a plugin to work.
+#
+package WWW::Mechanize;
+
+use Scalar::Util qw{ blessed };
+use Carp;
+
+=head1 WWW::Mechanize::request
+
+The method L<WWW::Mechanize/request> is overriden by a custom version that will
+invoke the original L<WWW::Mechanize/request> and then record the request.
+
+=cut
+
+{
+
+	no warnings qw{ redefine };
+
+	sub request {
+		my $self = shift;
+		my ($request, @args) = @_;
+
+		# Perform the actual HTTP request
+		my $response = $REQUEST_SUB->($self, $request, @args);
+
+		
+		# Write the request, response and contents
+		if (exists $self->{autowrite}) {
+		
+			my $autowrite = $self->{autowrite};
+			if (_is_an_autowrite($autowrite)) {
+				$autowrite->write_to_file($self, $request, $response);
+			}
+			else {
+				croak "Wrong type for member 'autowrite', got ", ref($autowrite) || 'a scalar';
+			}
+		}
+	
+		return $response;
+	}
+
+}
+
+
+=head1 WWW::Mechanize::autowrite
+
+This accessor returns the autowrite instance associated with this mechanize
+instance. The first time that this method is invoked it will create
+
+=cut
+
+sub autowrite {
+	my $self = shift;
+
+	# The first time that this accessor is invoked
+	
+	# set
+	if (@_) {
+		my $autowrite = shift;
+
+		if (! _is_an_autowrite($autowrite)) {
+			croak "Parameter must be an instance of WWW::Mechanize::Plugin::AutoWrite";
+		}
+
+		$self->{'autowrite'} = $autowrite;
+	}
+	# get
+	else {
+		if (! exists $self->{'autowrite'}) {
+			# Create an autowrite instance on the fly
+			$self->{'autowrite'} = WWW::Mechanize::Plugin::AutoWrite->new();
+		}
+	}
+	
+	return $self->{'autowrite'};
+}
+
+
+#
+# Returns true if the parameter is an instance of 
+# WWW::Mechanize::Plugin::AutoWrite.
+#
+sub _is_an_autowrite {
+	my ($autowrite) = @_;
+	return
+		blessed($autowrite) 
+		&& $autowrite->isa('WWW::Mechanize::Plugin::AutoWrite')
+	;
+}
+
+
 1;
 
+=head1 COMPATIBILITY
+
+The version 0.04 has a different API and is not backward compatible. This
+affects only the configuration of the plugin. The behaviour should be the same.
 
 =head1 SEE ALSO
 
@@ -275,11 +335,12 @@ L<http://search.cpan.org/perldoc?WWW::Mechanize::Plugin::Display>
 
 Jozef Kutej, E<lt>jkutej@cpan.orgE<gt>,
 
-Emmanuel Rodriguez, E<lt>emmanuel.rodriguez@gmail.comE<gt>
+Emmanuel Rodriguez, E<lt>potyl@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2007 by Jozef Kutej
+Copyright (C) 2008 by Jozef Kutej, Emmanuel Rodriguez
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
